"""Admin AJAX views for the Curate app
"""

import json

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.messages.storage.base import Message
from django.http import (
    HttpResponse,
    HttpResponseBadRequest,
)

import core_main_app.components.data.api as data_api

from core_main_app.commons.exceptions import DoesNotExist

from core_main_app.utils.labels import get_data_label, get_form_label

import core_curate_app.components.curate_data_structure.api as curate_data_structure_api


@staff_member_required
def delete_record_drafts(request, pk):
    """Delete record drafts.

    Args:
        request:
        pk:

    Returns:
    """
    try:
        # Get data
        data = data_api.get_by_id(pk, request.user)

        # delete curate data structures
        curate_data_structure_api.delete_curate_data_structures_by_data(
            data, request.user
        )

        message = Message(
            messages.SUCCESS,
            get_form_label() + " deleted.",
        )
    except DoesNotExist:
        message = Message(
            messages.ERROR,
            "It seems a "
            + get_data_label()
            + " is missing. Please refresh the page.",
        )
        return HttpResponseBadRequest(
            json.dumps({"message": message.message, "tags": message.tags}),
            content_type="application/json",
        )
    except Exception:
        message = Message(messages.ERROR, "A problem occurred while deleting.")
        return HttpResponseBadRequest(
            json.dumps({"message": message.message, "tags": message.tags}),
            content_type="application/json",
        )

    return HttpResponse(
        json.dumps({"message": message.message, "tags": message.tags}),
        content_type="application/json",
    )



""" Unit tests for views from `views.common.views`.
"""

from unittest.case import TestCase
from unittest.mock import patch, MagicMock

from django.test import RequestFactory

from core_curate_app.views.common.views import FormView, DataStructureMixin
from core_main_app.utils.labels import get_form_label
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestFormView(TestCase):
    """Test Form View"""

    def setUp(self):
        """setUp
        Returns:
        """
        self.factory = RequestFactory()
        self.user1 = create_mock_user(user_id="1", is_superuser=True)

    @patch("core_curate_app.components.curate_data_structure.api.get_by_id")
    def test_form_view_returns_error_when_xml_string_is_empty(
        self, mock_ds_get_by_id
    ):
        """test_form_view_returns_error_when_xml_string_is_empty


        Returns:


        """
        mock_ds = MagicMock()
        mock_ds.form_string = ""
        mock_ds_get_by_id.return_value = mock_ds

        request = self.factory.get("core_curate_view_form")

        request.user = self.user1
        response = FormView.as_view()(request, curate_data_structure_id=1)
        msg_error = (
            "The "
            + get_form_label()
            + " was not saved and cannot be displayed."
        )
        self.assertTrue(msg_error in response.content.decode())


class TestDataStructureMixin(TestCase):
    """TestDataStructureMixin"""

    def setUp(self):
        self.mixin_instance = DataStructureMixin()

    def test_get_modals(self):
        # Expected modals list
        expected_modals = [
            "core_main_app/common/modals/create_data_modal.html",
            "core_curate_app/user/data-entry/modals/switch_to_form_editor.html",
        ]

        # Get the actual modals from the mixin
        actual_modals = self.mixin_instance._get_modals()

        # Assert that the actual modals match the expected modals
        self.assertEqual(actual_modals, expected_modals)



""" Unit Test Access Control
"""

from unittest.case import TestCase

from core_curate_app.access_control import api as access_control_api
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestAccessControlCheckCanWrite(TestCase):
    """Test Access Control Check Can Write"""

    def setUp(self):
        """setUp"""
        self.anonymous_user = create_mock_user(user_id=None, is_anonymous=True)
        self.user1 = create_mock_user(user_id="1")
        self.superuser = create_mock_user(user_id="1", is_superuser=True)

    def test_access_control_check_can_write_as_none_raises_access_control_error(
        self,
    ):
        """test_access_control_check_can_write_as_none_raises_access_control_error

        Returns:

        """
        # Arrange
        data_structure = CurateDataStructure(
            user="1",
            template=Template(),
            name="name",
        )

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.check_can_write(data_structure, user=None)

    def test_access_control_check_can_write_as_anonymous_raises_access_control_error(
        self,
    ):
        """test_access_control_check_can_write_as_anonymous_raises_access_control_error

        Returns:

        """
        # Arrange
        data_structure = CurateDataStructure(
            user="1",
            template=Template(),
            name="name",
        )

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.check_can_write(
                data_structure, user=self.anonymous_user
            )

    def test_access_control_check_can_write_as_user_raises_access_control_error(
        self,
    ):
        """test_access_control_check_can_write_as_user_raises_access_control_error

        Returns:

        """
        # Arrange
        data_structure = CurateDataStructure(
            user="2",
            template=Template(),
            name="name",
        )

        # Act # Assert
        with self.assertRaises(AccessControlError):
            access_control_api.check_can_write(data_structure, user=self.user1)

    def test_access_control_check_can_write_as_superuser_returns_none(
        self,
    ):
        """test_access_control_check_can_write_as_superuser_returns_none

        Returns:

        """
        # Arrange
        data_structure = CurateDataStructure(
            user="2",
            template=Template(),
            name="name",
        )

        # Act # Assert
        access_control_api.check_can_write(data_structure, user=self.superuser)

    def test_access_control_check_can_write_as_owner_returns_none(
        self,
    ):
        """test_access_control_check_can_write_as_owner_returns_none

        Returns:

        """
        # Arrange
        data_structure = CurateDataStructure(
            user="1",
            template=Template(),
            name="name",
        )

        # Act # Assert
        access_control_api.check_can_write(data_structure, user=self.superuser)



""" Unit tests for views from `views.user.ajax`.
"""

import json
from unittest import TestCase
from unittest.mock import patch, Mock, MagicMock

from django.http import HttpResponseBadRequest
from django.test import RequestFactory

from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_curate_app.views.user import ajax as curate_user_ajax
from core_curate_app.views.user import views as curate_user_views
from core_main_app.commons.exceptions import DoesNotExist, JSONError
from core_main_app.components.template.models import Template
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCancelChangesView(TestCase):
    """Unit tests for `cancel_changes` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_cancel_changes")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch("core_curate_app.views.user.views.render_form")
    @patch("core_curate_app.views.user.views.generate_root_element")
    @patch("core_curate_app.components.curate_data_structure.api.get_by_id")
    def test_cancel_changes_return_http_response_with_xsd_form(
        self,
        mock_curate_data_structure_get_by_id,
        mock_generate_root_element,
        mock_render_form,
    ):
        """test_cancel_changes_return_http_response_with_xsd_form"""
        mock_data = MagicMock(
            data=None, form_string=None, template=_get_template()
        )
        mock_curate_data_structure_get_by_id.return_value = mock_data

        mock_generate_root_element.return_value = MagicMock()
        mock_render_form.return_value = "form"

        response = curate_user_ajax.cancel_changes(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn("xsdForm", response.content.decode())

    @patch("core_curate_app.components.curate_data_structure.api.get_by_id")
    def test_cancel_changes_return_http_response_with_content(
        self,
        mock_curate_data_structure_get_by_id,
    ):
        """test_cancel_changes_return_http_response_with_content"""
        mock_data = MagicMock(
            form_string="test", data=None, template=_get_json_template()
        )
        mock_curate_data_structure_get_by_id.return_value = mock_data

        response = curate_user_ajax.cancel_changes(
            self.request,
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn("content", response.content.decode())

    @patch("core_curate_app.views.user.views.render_form")
    @patch("core_curate_app.views.user.views.generate_root_element")
    @patch("core_curate_app.components.curate_data_structure.api.get_by_id")
    def test_cancel_changes_with_empty_string_form_returns_http_200(
        self,
        mock_curate_data_structure_get_by_id,
        mock_generate_root_element,
        mock_render_form,
    ):
        """test_cancel_changes_with_empty_string_form_returns_http_200"""
        mock_data = MagicMock(template=_get_template())
        mock_curate_data_structure_get_by_id.return_value = mock_data

        mock_generate_root_element.return_value = MagicMock()
        mock_render_form.return_value = "form"

        response = curate_user_ajax.cancel_changes(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn("xsdForm", response.content.decode())

    @patch("core_curate_app.views.user.views.render_form")
    @patch("core_curate_app.views.user.views.generate_root_element")
    @patch("core_curate_app.components.curate_data_structure.api.get_by_id")
    def test_cancel_changes_with_string_form_returns_http_200(
        self,
        mock_curate_data_structure_get_by_id,
        mock_generate_root_element,
        mock_render_form,
    ):
        """test_cancel_changes_with_empty_string_form_returns_http_200"""
        mock_data = MagicMock(string_form="test", template=_get_template())
        mock_curate_data_structure_get_by_id.return_value = mock_data

        mock_generate_root_element.return_value = MagicMock()
        mock_render_form.return_value = "form"

        response = curate_user_ajax.cancel_changes(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn("xsdForm", response.content.decode())

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_cancel_changes_returns_http_bad_request_when_template_format_is_not_supported(
        self, mock_curate_data_structure_api
    ):
        """test_cancel_changes_with_xsd_template_returns_http_200"""
        mock_curate_data_structure = Mock(
            template=Template(format="BAD_FORMAT")
        )
        mock_curate_data_structure.data = None
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_curate_data_structure_api.delete.return_value = None

        response = curate_user_ajax.cancel_changes(self.request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")


class TestCancelFormView(TestCase):
    """Unit tests for `cancel_form` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_cancel_form")
        self.request.POST = {"id": 1}
        self.request.user = user1

    def test_request_post_id_error_returns_http_bad_request(self):
        """test_request_post_id_error_returns_http_bad_request"""
        del self.request.POST["id"]
        response = curate_user_ajax.cancel_form(self.request)

        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_get_by_id_called(
        self, mock_curate_data_structure_api
    ):
        """test_curate_data_structure_get_by_id_called"""
        curate_user_ajax.cancel_form(self.request)
        mock_curate_data_structure_api.get_by_id.assert_called_with(
            self.request.POST["id"], self.request.user
        )

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_get_by_id_error_returns_http_bad_request(
        self, mock_curate_data_structure_api
    ):
        """test_curate_data_structure_get_by_id_error_returns_http_bad_request"""
        mock_curate_data_structure_api.get_by_id.side_effect = Exception(
            "mock_get_by_id_exception"
        )
        response = curate_user_ajax.cancel_form(self.request)
        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_ajax, "lock_api")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_data_locked_if_data_linked_to_curate_data_structure(
        self, mock_curate_data_structure_api, mock_lock_api
    ):
        """test_data_locked_if_data_linked_to_curate_data_structure"""
        mock_curate_data_structure = Mock()
        mock_curate_data_structure.data = Mock()
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        curate_user_ajax.cancel_form(self.request)

        mock_lock_api.remove_lock_on_object.assert_called_with(
            mock_curate_data_structure.data, self.request.user
        )

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_datastructure_delete_called(
        self, mock_curate_data_structure_api
    ):
        """test_curate_datastructure_delete_called"""
        mock_curate_data_structure = Mock()
        mock_curate_data_structure.data = None
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        curate_user_ajax.cancel_form(self.request)

        mock_curate_data_structure_api.delete.assert_called_with(
            mock_curate_data_structure, self.request.user
        )

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_delete_error_returns_http_bad_request(
        self, mock_curate_data_structure_api
    ):
        """test_curate_data_structure_delete_error_returns_http_bad_request"""
        mock_curate_data_structure = Mock()
        mock_curate_data_structure.data = None
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_curate_data_structure_api.delete.side_effect = Exception(
            "mock_delete_exception"
        )

        response = curate_user_ajax.cancel_form(self.request)
        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_ajax, "messages")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_add_message_called(
        self, mock_curate_data_structure_api, mock_messages
    ):
        """test_add_message_called"""
        mock_curate_data_structure = Mock()
        mock_curate_data_structure.data = None
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_curate_data_structure_api.delete.return_value = None

        curate_user_ajax.cancel_form(self.request)
        mock_messages.add_message.assert_called()


class TestSaveFormView(TestCase):
    """Unit tests for `save_form` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_save_form")
        self.request.POST = {"id": 1}
        self.request.user = user1

    def test_request_post_id_error_returns_http_bad_request(self):
        """test_request_post_id_error_returns_http_bad_request"""
        del self.request.POST["id"]

        response = curate_user_ajax.save_form(self.request)
        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_api_get_by_id_called(
        self, mock_curate_data_structure_api
    ):
        """test_curate_data_structure_api_get_by_id_called"""
        curate_user_ajax.save_form(self.request)

        mock_curate_data_structure_api.get_by_id.assert_called_with(
            self.request.POST["id"], self.request.user
        )

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_api_get_by_id_error_returns_http_bad_request(
        self, mock_curate_data_structure_api
    ):
        """test_curate_data_structure_api_get_by_id_error_returns_http_bad_request"""
        mock_curate_data_structure_api.get_by_id.side_effect = Exception(
            "mock_get_by_id_exception"
        )

        response = curate_user_ajax.save_form(self.request)
        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_render_xml_called(
        self, mock_curate_data_structure_api, mock_render_xml
    ):
        """test_render_xml_called"""
        mock_curate_data_structure = CurateDataStructure(
            form_string="text", template=Template(format=Template.XSD)
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        curate_user_ajax.save_form(self.request)

        mock_render_xml.assert_called_with(
            self.request,
            mock_curate_data_structure.data_structure_element_root,
        )

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_render_xml_error_returns_http_bad_request(
        self, mock_curate_data_structure_api, mock_render_xml
    ):
        """test_render_xml_error_returns_http_bad_request"""
        mock_curate_data_structure = Mock()
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_render_xml.side_effect = Exception("mock_render_xml_exception")

        response = curate_user_ajax.save_form(self.request)
        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_api_upsert_called(
        self, mock_curate_data_structure_api, mock_render_xml
    ):
        """test_curate_data_structure_api_upsert_called"""
        mock_curate_data_structure = CurateDataStructure(
            form_string="text", template=Template(format=Template.XSD)
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_render_xml.return_value = Mock()

        curate_user_ajax.save_form(self.request)

        mock_curate_data_structure_api.upsert.assert_called_with(
            mock_curate_data_structure, self.request.user
        )

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_curate_data_structure_api_upsert_error_returns_http_bad_request(
        self, mock_curate_data_structure_api, mock_render_xml
    ):
        """test_curate_data_structure_api_upsert_error_returns_http_bad_request"""
        mock_curate_data_structure = Mock()
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_render_xml.return_value = Mock()
        mock_curate_data_structure_api.upsert.side_effect = Exception(
            "mock_upsert_exception"
        )

        response = curate_user_ajax.save_form(self.request)
        self.assertIsInstance(response, HttpResponseBadRequest)

    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_form_with_xsd_template_returns_http_200(
        self, mock_curate_data_structure_api, mock_render_xml
    ):
        """test_save_form_with_xsd_template_returns_http_200"""
        mock_template = _get_template()
        mock_curate_data_structure_api.get_by_id.return_value = MagicMock(
            template=mock_template, data=None, name="title"
        )
        mock_render_xml.return_value = Mock()
        mock_curate_data_structure_api.upsert.return_value = None

        response = curate_user_ajax.save_form(self.request)
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_form_with_json_template_returns_http_200(
        self, mock_curate_data_structure_api
    ):
        """test_save_form_with_json_template_returns_http_200"""
        mock_template = _get_json_template()
        mock_curate_data_structure_api.get_by_id.return_value = MagicMock(
            template=mock_template, data=None, name="title"
        )
        mock_curate_data_structure_api.upsert.return_value = None

        response = curate_user_ajax.save_form(self.request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue("saved." in response.content.decode("utf-8"))

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_form_with_well_formed_content_returns_http_200(
        self, mock_curate_data_structure_api
    ):
        """test_save_form_with_well_formed_content_returns_http_200"""
        mock_curate_data_structure = CurateDataStructure(
            form_string="text", template=Template(format=Template.XSD)
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        self.request.POST = {"id": 1, "form_string": "<text></text>"}
        response = curate_user_ajax.save_form(self.request)
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_form_with_not_well_formed_content_returns_error(
        self, mock_curate_data_structure_api
    ):
        """test_save_form_with_not_well_formed_content_returns_error"""
        mock_curate_data_structure = CurateDataStructure(
            form_string="text", template=Template(format=Template.XSD)
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        self.request.POST = {"id": 1, "form_string": "</text>"}
        response = curate_user_ajax.save_form(self.request)
        self.assertEqual(response.status_code, 400)
        self.assertTrue(
            "Content is not well formatted XML." in response.content.decode()
        )


class TestGenerateChoiceView(TestCase):
    """Unit tests for the `generate_choice` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_generate_choice")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_generate_choice_return_http_bad_request_if_not_xsd_template(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_generate_choice_return_http_bad_request_if_not_xsd_template"""
        mock_template = _get_json_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_ajax.generate_choice(
            self.request, curate_data_structure_id=1
        )

        self.assertTrue(mock_template_get_by_id.called)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_generate_choice_with_error_returns_http_bad_request(
        self, mock_curate_data_structure_api
    ):
        """test_generate_choice_with_error_returns_http_bad_request"""
        mock_curate_data_structure_api.side_effect = DoesNotExist("error")

        response = curate_user_ajax.generate_choice(
            self.request, curate_data_structure_id=1
        )

        self.assertEqual(response.status_code, 400)

    @patch(
        "core_parser_app.tools.parser.parser.XSDParser.generate_choice_absent"
    )
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_generate_choice_returns_html_form(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_generate_choice_absent,
    ):
        """test_generate_choice_returns_html_form"""
        mock_template = _get_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template
        mock_generate_choice_absent.return_value = "form"

        response = curate_user_ajax.generate_choice(
            self.request, curate_data_structure_id=1
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"form")


class TestGenerateElementView(TestCase):
    """Unit tests for `generate_element` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_generate_element")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_generate_element_return_http_bad_request_if_not_xsd_template(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_generate_element_return_http_bad_request_if_not_xsd_template"""
        mock_template = _get_json_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_ajax.generate_element(
            self.request, curate_data_structure_id=1
        )

        self.assertTrue(mock_template_get_by_id.called)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")

    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_generate_element_with_error_returns_http_bad_request(
        self, mock_curate_data_structure_api
    ):
        """test_generate_element_with_error_returns_http_bad_request"""
        mock_curate_data_structure_api.side_effect = DoesNotExist("error")

        response = curate_user_ajax.generate_element(
            self.request, curate_data_structure_id=1
        )

        self.assertEqual(response.status_code, 400)

    @patch(
        "core_parser_app.tools.parser.parser.XSDParser.generate_element_absent"
    )
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_generate_element_returns_html_form(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_generate_choice_absent,
    ):
        """test_generate_element_returns_html_form"""
        mock_template = _get_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template
        mock_generate_choice_absent.return_value = "form"

        response = curate_user_ajax.generate_element(
            self.request, curate_data_structure_id=1
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b"form")


class TestClearFieldsView(TestCase):
    """Unit tests for `clear_fields` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_generate_element")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_clear_fields_return_http_bad_request_if_not_xsd_template(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_clear_fields_return_http_bad_request_if_not_xsd_template"""
        mock_template = _get_json_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_ajax.clear_fields(
            self.request,
        )

        self.assertTrue(mock_template_get_by_id.called)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")

    @patch("core_curate_app.views.user.views.render_form")
    @patch("core_curate_app.views.user.views.generate_form")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_clear_fields_returns_form(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_generate_form,
        mock_render_form,
    ):
        """test_clear_fields_returns_form"""
        mock_template = _get_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template
        mock_generate_form.return_value = MagicMock()
        mock_render_form.return_value = "form"

        response = curate_user_ajax.clear_fields(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn("xsdForm", response.content.decode())


class TestValidateFormView(TestCase):
    """Unit tests for `validate_form` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_validate_form")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_validate_form_returns_http_bad_request_when_template_is_not_supported(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_validate_form_return_http_bad_request_when_template_is_not_supported"""
        mock_template = _get_json_template()
        mock_template.format = "mock_unsupported_format"
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_ajax.validate_form(
            self.request,
        )

        self.assertTrue(mock_template_get_by_id.called)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")

    @patch.object(curate_user_views, "render_xml")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_validate_form_returns_errors(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_render_xml,
    ):
        """test_validate_form_returns_errors"""
        mock_template = _get_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template
        mock_render_xml.return_value = "<root></root>"

        response = curate_user_ajax.validate_form(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn("errors", response.content.decode())

    @patch.object(curate_user_views, "render_xml")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_invalid_xml_returns_errors(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_render_xml,
    ):
        """test_invalid_xml_returns_errors"""
        mock_template = _get_template()
        mock_curate_data_structure_api.return_value = MagicMock(
            template=mock_template
        )
        mock_template_get_by_id.return_value = mock_template
        mock_render_xml.return_value = "<root></ro"

        response = curate_user_ajax.validate_form(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn("errors", response.content.decode())

    @patch.object(curate_user_ajax, "main_json_utils")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_validate_json_form_calls_validate_json_data(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_json_utils,
    ):
        """test_validate_json_form_calls_validate_json_data"""
        mock_template = _get_json_template()
        mock_curate_data_structure = MagicMock(
            template=mock_template, form_string=json.dumps({"root": "value"})
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_template_get_by_id.return_value = mock_template

        curate_user_ajax.validate_form(
            self.request,
        )

        mock_json_utils.validate_json_data.assert_called_with(
            mock_curate_data_structure.form_string, mock_template.content
        )

    @patch.object(curate_user_ajax, "main_json_utils")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_invalid_json_form_returns_errors(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_json_utils,
    ):
        """test_invalid_json_form_returns_errors"""
        mock_template = _get_json_template()
        mock_curate_data_structure = MagicMock(
            template=mock_template, form_string=json.dumps({"root": "value"})
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_template_get_by_id.return_value = mock_template
        mock_json_errors = ["error1", "error2"]
        mock_json_utils.validate_json_data.side_effect = JSONError(
            mock_json_errors
        )

        response = curate_user_ajax.validate_form(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content), {"errors": mock_json_errors}
        )

    @patch.object(curate_user_ajax, "main_json_utils")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_valid_json_form_returns_empty(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_json_utils,
    ):
        """test_invalid_json_form_returns_errors"""
        mock_template = _get_json_template()
        mock_curate_data_structure = MagicMock(
            template=mock_template, form_string=json.dumps({"root": "value"})
        )
        mock_curate_data_structure_api.get_by_id.return_value = (
            mock_curate_data_structure
        )
        mock_template_get_by_id.return_value = mock_template
        mock_json_utils.validate_json_data.return_value = None

        response = curate_user_ajax.validate_form(
            self.request,
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(json.loads(response.content), {})


class TestSaveDataView(TestCase):
    """Unit tests for `save_data` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_save_data")
        self.request.POST = {"id": 1}
        self.request.user = user1

    @patch.object(curate_user_ajax, "messages")
    @patch("core_main_app.components.data.api.upsert")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_views, "render_xml")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_data_returns_http_response_if_xsd_template(
        self,
        mock_curate_data_structure_api,
        mock_render_xml,
        mock_template_get_by_id,
        mock_data_upsert,
        mock_messages,
    ):
        """test_save_data_returns_http_response_if_xsd_template"""
        mock_template = _get_template()
        mock_curate_data_structure_api.get_by_id.return_value = MagicMock(
            template=mock_template, data=None, name="title"
        )
        mock_curate_data_structure_api.delete.return_value = None
        mock_render_xml.return_value = "<root></root>"
        mock_template_get_by_id.return_value = mock_template
        mock_data_upsert.return_value = MagicMock(id=1)

        response = curate_user_ajax.save_data(
            self.request,
        )

        self.assertTrue(mock_curate_data_structure_api.get_by_id.called)
        self.assertTrue(mock_curate_data_structure_api.delete.called)
        self.assertTrue(mock_template_get_by_id.called)
        self.assertTrue(mock_data_upsert.called)
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_ajax, "messages")
    @patch("core_main_app.components.data.api.upsert")
    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_data_returns_http_response_if_json_template(
        self,
        mock_curate_data_structure_api,
        mock_template_get_by_id,
        mock_data_upsert,
        mock_messages,
    ):
        """test_save_data_returns_http_response_if_json_template"""
        mock_template = _get_json_template()
        mock_curate_data_structure_api.get_by_id.return_value = MagicMock(
            template=mock_template, data=None, name="title"
        )
        mock_curate_data_structure_api.delete.return_value = None
        mock_template_get_by_id.return_value = mock_template
        mock_data_upsert.return_value = MagicMock(id=1)

        response = curate_user_ajax.save_data(
            self.request,
        )

        self.assertTrue(mock_curate_data_structure_api.get_by_id.called)
        self.assertTrue(mock_curate_data_structure_api.delete.called)
        self.assertTrue(mock_template_get_by_id.called)
        self.assertTrue(mock_data_upsert.called)
        self.assertEqual(response.status_code, 200)

    @patch.object(curate_user_ajax, "messages")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_save_data_returns_http_bad_request_when_template_is_not_supported(
        self,
        mock_curate_data_structure_api,
        mock_messages,
    ):
        """test_save_data_returns_http_bad_request_when_template_is_not_supported"""
        mock_curate_data_structure_api.get_by_id.return_value = MagicMock(
            template=Template(format="BAD_FORMAT"), data=None, name="title"
        )
        mock_curate_data_structure_api.delete.return_value = None

        response = curate_user_ajax.save_data(
            self.request,
        )

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")


class TestStartCurateGetView(TestCase):
    """Unit tests for `_start_curate_get` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.get("core_curate_start")
        self.request.GET = {"template_id": 1}
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_start_curate_get_returns_http_response(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_start_curate_get_returns_http_response"""
        mock_template = _get_template()
        mock_curate_data_structure_api.get_all_by_user_id_and_template_id_with_no_data.return_value = MagicMock(
            template=mock_template,
        )
        mock_curate_data_structure_api.delete.return_value = None
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_ajax._start_curate_get(
            self.request,
        )

        self.assertEqual(response.status_code, 200)


class TestStartCuratePostView(TestCase):
    """Unit tests for `_start_curate_post` method."""

    def setUp(self):
        """setUp"""
        factory = RequestFactory()
        user1 = create_mock_user(user_id="1", has_perm=True)

        self.request = factory.post("core_curate_start", {"file": Mock()})
        self.request.POST = {
            "template_id": 1,
            "curate_form": "new",
            "template_format": "XSD",
            "document_name": "test",
        }
        self.request.user = user1

    @patch("core_main_app.components.template.api.get_by_id")
    @patch.object(curate_user_ajax, "curate_data_structure_api")
    def test_start_curate_post_returns_http_bad_response_when_bad_template_format(
        self, mock_curate_data_structure_api, mock_template_get_by_id
    ):
        """test_start_curate_post_returns_http_bad_response_when_bad_template_format"""
        mock_template = Template(format=None)
        mock_curate_data_structure_api.get_all_by_user_id_and_template_id_with_no_data.return_value = MagicMock(
            template=mock_template,
        )
        mock_curate_data_structure_api.delete.return_value = None
        mock_template_get_by_id.return_value = mock_template

        response = curate_user_ajax._start_curate_post(
            self.request,
        )

        self.assertTrue(mock_template_get_by_id.called)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b"Template format not supported.")

    @patch.object(curate_user_ajax, "messages")
    @patch("core_main_app.components.data.api.upsert")
    @patch("core_main_app.views.common.views.read_xsd_file")
    @patch("core_main_app.utils.xml.is_well_formed_xml")
    @patch("core_main_app.components.template.api.get_by_id")
    def test_start_curate_post_upload_returns_http_response(
        self,
        mock_template_get_by_id,
        mock_is_well_formed,
        mock_read_xsd_file,
        mock_data_upsert,
        mock_messages,
    ):
        """test_start_curate_get_returns_http_response"""
        self.request.POST["curate_form"] = "upload"
        self.request.POST["direct_upload"] = True
        mock_template = _get_template()
        mock_template_get_by_id.return_value = mock_template
        mock_is_well_formed.return_value = True
        mock_read_xsd_file.return_value = "<root></root>"
        mock_data_upsert.return_value = None

        response = curate_user_ajax._start_curate_post(
            self.request,
        )

        self.assertEqual(response.status_code, 200)


def _get_json_template():
    """Get JSON template

    Returns:

    """
    template = Template()
    template.format = Template.JSON
    template.id_field = 1
    template.content = "{}"
    return template


def _get_template():
    """Get XSD template

    Returns:

    """
    template = Template()
    template.id_field = 1
    xsd = (
        '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
        '<xs:element name="tag"></xs:element></xs:schema>'
    )
    template.content = xsd
    template.format = Template.XSD
    return template



"""
    Common views
"""

import json

from django.contrib import messages
from django.http import (
    HttpResponseBadRequest,
    HttpResponseForbidden,
    HttpResponse,
)
from django.urls import reverse
from django.utils.html import escape as html_escape
from django.utils.translation import gettext as _

from core_curate_app.components.curate_data_structure import (
    api as data_structure_api,
)
from core_main_app.commons import exceptions

from core_main_app.utils import file as main_file_utils
from core_curate_app.views.user.views import _get_curate_data_structure_by_id
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import (
    DoesNotExist,
    JSONError,
)
from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.components.template import api as template_api
from core_main_app.settings import MAX_DOCUMENT_EDITING_SIZE
from core_main_app.utils.assets_combiner import join_assets

from core_main_app.utils.labels import get_data_label, get_form_label
from core_main_app.views.common.views import CommonView, XmlEditor, JSONEditor
from core_curate_app.access_control import api as acl_api


class FormView(CommonView):
    """
    Form View
    """

    template = "core_curate_app/user/detail.html"

    def get(self, request, *args, **kwargs):
        """Get the web page of a given form

        Args:
            request:

        Returns:
        """
        try:
            # get data structure
            curate_data_structure = _get_curate_data_structure_by_id(
                kwargs["curate_data_structure_id"], request
            )

            # TODO: fix with the rework on the curate workflow
            if (
                curate_data_structure.form_string is None
                or curate_data_structure.form_string == ""
            ):
                raise Exception(
                    "The "
                    + _("form_label")
                    + " was not saved and cannot be displayed."
                )

            # Set the assets
            assets = {
                "js": [
                    {
                        "path": "core_main_app/common/js/XMLTree.js",
                        "is_raw": False,
                    },
                    {
                        "path": "core_main_app/user/js/data/detail.js",
                        "is_raw": False,
                    },
                ],
                "css": ["core_main_app/common/css/XMLTree.css"],
            }

            # Set the context
            context = {
                "data_structure": curate_data_structure,
                "page_title": "View Form",
            }

            return self.common_render(
                request, self.template, assets=assets, context=context
            )
        except Exception as exception:
            assets = {
                "js": [
                    {
                        "path": "core_main_app/user/js/data/detail.js",
                        "is_raw": False,
                    },
                ]
            }
            template = "core_main_app/common/commons/error.html"
            if self.administration:
                template = (
                    "core_main_app/admin/commons/errors/errors_wrapper.html"
                )

            return self.common_render(
                request,
                template,
                context={"error": str(exception), "page_title": "Error"},
                assets=assets,
            )


class DataStructureMixin:
    """DataStructureMixin"""

    object_name = get_form_label()

    def _get_object(self, request):
        """get object

        Args:
            request

        Returns:
        """
        return data_structure_api.get_by_id(request.GET["id"], request.user)

    def _check_permission(self, data_structure, request):
        """check user permission
        Args:
            data_structure:
            request:
        Returns:
        """
        return acl_api.check_can_write(data_structure, request.user)

    def _check_size(self, data_structure):
        """check content size

        Args:
            data_structure:

        Returns:
        """

        if (
            main_file_utils.get_byte_size_from_string(
                data_structure.form_string
            )
            > MAX_DOCUMENT_EDITING_SIZE
        ):
            raise exceptions.DocumentEditingSizeError(
                "The file is too large (MAX_DOCUMENT_EDITING_SIZE)."
            )

    def _get_assets(self):
        """get assets

        Args:

        Returns:
        """
        assets = {
            "js": [
                {
                    "path": "core_curate_app/user/js/text_editor.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_curate_app/user/js/switch_editor.js",
                    "is_raw": False,
                },
            ]
        }

        return assets

    def _get_modals(self):
        """get modals

        Args:

        Return:
        """

        modals = [
            "core_main_app/common/modals/create_data_modal.html",
            "core_curate_app/user/data-entry/modals/switch_to_form_editor.html",
        ]
        return modals

    def save(self, *args, **kwargs):
        """Save as data
        Args:
            *args:
            **kwargs:
        Returns:
        """
        try:
            request = kwargs.get("request")
            content = request.POST["content"].strip()
            data_structure_id = request.POST["document_id"]
            data_structure = data_structure_api.get_by_id(
                data_structure_id, request.user
            )
            if data_structure.data is not None:
                data = data_structure.data
            else:
                # create new data
                data = Data()
                data.title = data_structure.name
                template = template_api.get_by_id(
                    str(data_structure.template.id), request=request
                )
                data.template = template
                data.user_id = str(request.user.id)

            # set content
            data.content = content
            # save data
            data_api.upsert(data, request)
            data_structure_api.delete(data_structure, request.user)
            messages.add_message(
                request,
                messages.SUCCESS,
                get_data_label().capitalize() + " saved.",
            )
            return HttpResponse(
                json.dumps({"url": reverse("core_curate_index")}),
                "application/javascript",
            )
        except AccessControlError as ace:
            return HttpResponseForbidden(html_escape(str(ace)))
        except DoesNotExist as dne:
            return HttpResponseBadRequest(html_escape(str(dne)))
        except JSONError as json_error:
            return HttpResponseBadRequest(
                json.dumps(
                    [
                        html_escape(str(message))
                        for message in json_error.message_list
                    ]
                )
            )
        except Exception as e:
            return HttpResponseBadRequest(html_escape(str(e)))


class DataStructureXMLEditor(DataStructureMixin, XmlEditor):
    """Data Structure XML Editor View"""

    def _prepare_context(self, data_structure):
        """prepare context
        Args:
            data_structure:

        Returns:
        """
        return self.get_context(
            data_structure, data_structure.name, data_structure.form_string
        )

    def _get_assets(self):
        """get assets from DataStructureMixin & XmlEditor

        Returns:

        """
        assets_from_mixin = DataStructureMixin._get_assets(self)
        assets_from_editor = XmlEditor._get_assets(self)

        return join_assets(assets_from_mixin, assets_from_editor)

    def _get_modals(self):
        """get modals from DataStructureMixin & XmlEditor

        Returns:

        """
        modals_from_mixin = DataStructureMixin._get_modals(self)
        modals_from_editor = XmlEditor._get_modals(self)

        return modals_from_mixin + modals_from_editor


class DataStructureJSONEditor(DataStructureMixin, JSONEditor):
    """Data Structure JSON Editor View"""

    def _prepare_context(self, data_structure):
        """prepare context

        Args:
            data_structure:

        Returns:
        """
        return self.get_context(
            data_structure, data_structure.name, data_structure.form_string
        )

    def _get_assets(self):
        """get assets from DataStructureMixin & XmlEditor

        Returns:

        """
        assets_from_mixin = DataStructureMixin._get_assets(self)
        assets_from_editor = JSONEditor._get_assets(self)

        return join_assets(assets_from_mixin, assets_from_editor)

    def _get_modals(self):
        """get modals from DataStructureMixin & JSONEditor

        Returns:

        """
        modals_from_mixin = DataStructureMixin._get_modals(self)
        modals_from_editor = JSONEditor._get_modals(self)

        return modals_from_mixin + modals_from_editor



""" ACL Test Data Structure
"""

from django.contrib.auth.models import AnonymousUser
from tests.components.curate_data_structure.fixtures.fixtures import (
    DataStructureFixtures2,
    DataStructureRootElementsFixtures,
)

import core_curate_app.components.curate_data_structure.api as curate_data_structure_api
from core_curate_app.components.curate_data_structure.models import (
    CurateDataStructure,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.utils.integration_tests.integration_base_test_case import (
    IntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_parser_app.components.data_structure.models import (
    DataStructureElement,
)

fixture_data_structure = DataStructureFixtures2()
fixture_data_structure_root_element = DataStructureRootElementsFixtures()


class TestGetCurateDataStructuresByData(IntegrationBaseTestCase):
    """
    Test Get Curate Data Structures By Data
    """

    fixture = fixture_data_structure

    def test_get_all_curate_data_structures_by_data_returns_data_structure_list(
        self,
    ):
        """
        test_get_all_curate_data_structures_by_data_returns_data_structure_list

        Returns:

        """
        mock_user = create_mock_user(
            self.fixture.data_structure_1.user, is_superuser=True
        )
        result = (
            curate_data_structure_api.get_all_curate_data_structures_by_data(
                self.fixture.data, mock_user
            )
        )
        for data_structure in result:
            self.assertTrue(isinstance(data_structure, CurateDataStructure))

    def test_get_all_curate_data_structures_by_data_returns_empty_list_when_data_has_no_data_structure(
        self,
    ):
        """
        test_get_all_curate_data_structures_by_data_returns_empty_list_when_data_has_no_data_structure

        Returns:

        """
        mock_user = create_mock_user("0", is_superuser=True)
        result = (
            curate_data_structure_api.get_all_curate_data_structures_by_data(
                self.fixture.data_without_draft, mock_user
            )
        )
        self.assertEqual(list(result), [])

    def test_get_all_curate_data_structures_by_data_as_user_raise_error(
        self,
    ):
        """
        test_get_all_curate_data_structures_by_data_as_user_raise_error

        Returns:

        """
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all_curate_data_structures_by_data(
                self.fixture.data_multiple_drafts, mock_user
            )

    def test_get_all_curate_data_structures_by_data_as_anonymous_raise_error(
        self,
    ):
        """
        test_get_all_curate_data_structures_by_data_as_anonymous_raise_error

        Returns:

        """

        # Act # Assert
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.get_all_curate_data_structures_by_data(
                self.fixture.data_multiple_drafts, AnonymousUser()
            )


class TestDeleteCurateDataStructuresByData(IntegrationBaseTestCase):
    """
    Test Delete Curate Data Structures By Data
    """

    fixture = fixture_data_structure

    def test_delete_curate_data_structures_by_data_deletes_curate_data_structures(
        self,
    ):
        """
        test_delete_curate_data_structures_by_data_deletes_curate_data_structures

        Returns:

        """
        mock_user = create_mock_user("1", is_superuser=True, is_staff=True)
        curate_data_structure_api.delete_curate_data_structures_by_data(
            self.fixture.data, mock_user
        )

    def test_delete_curate_data_structures_by_data_returns_none(
        self,
    ):
        """
        test_delete_curate_data_structures_by_data_returns_none

        Returns:

        """
        mock_user = create_mock_user("0", is_superuser=True)
        curate_data_structure_api.delete_curate_data_structures_by_data(
            self.fixture.data_without_draft, mock_user
        )

    def test_delete_curate_data_structures_by_data_raise_error(
        self,
    ):
        """
        test_delete_curate_data_structures_by_data_raise_error

        Returns:

        """
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.delete_curate_data_structures_by_data(
                self.fixture.data_multiple_drafts, mock_user
            )

    def test_delete_curate_data_structures_by_data_raise_error_as_anonymous_raise_error(
        self,
    ):
        """
        test_delete_curate_data_structures_by_data_raise_error_as_anonymous_raise_error

        Returns:

        """

        # Act # Assert
        with self.assertRaises(AccessControlError):
            curate_data_structure_api.delete_curate_data_structures_by_data(
                self.fixture.data_multiple_drafts, AnonymousUser()
            )


class TestCurateDataStructureUpdateDataStructureRoot(IntegrationBaseTestCase):
    """
    Test Curate Data Structure Update Data Structure Root
    """

    fixture = fixture_data_structure_root_element

    def test_update_empty_data_structure_root_updates_data_structure(self):
        """
        test_update_empty_data_structure_root_updates_data_structure

        Returns:

        """
        data_structure = self.fixture.data_structure_1
        mock_user = create_mock_user(self.fixture.data_structure_1.user)

        result = curate_data_structure_api.update_data_structure_root(
            data_structure, self.fixture.data_structure_root_element, mock_user
        )
        self.assertTrue(isinstance(result, CurateDataStructure))
        self.assertNotEquals(result.data_structure_element_root, None)
        self.assertEquals(
            result.data_structure_element_root,
            self.fixture.data_structure_root_element,
        )


class TestDeleteCurateDataStructure(IntegrationBaseTestCase):
    """
    Test Delete Curate Data Structure
    """

    fixture = fixture_data_structure_root_element

    def test_delete_data_structure_without_data_structure_root_deletes_data_structure(
        self,
    ):
        """
        test_delete_data_structure_with_empty_data_structure_root_deletes_data_structure

        Returns:

        """
        mock_user = create_mock_user(self.fixture.data_structure_1.user)
        self.fixture.data_structure_1.delete()
        with self.assertRaises(DoesNotExist):
            curate_data_structure_api.get_by_id(
                self.fixture.data_structure_1.id, mock_user
            )

    def test_delete_data_structure_with_data_structure_root_deletes_data_structure_and_element_root(
        self,
    ):
        """
        test_delete_data_structure_with_empty_data_structure_root_deletes_data_structure

        Returns:

        """
        mock_user = create_mock_user(self.fixture.data_structure_1.user)

        new_data_structure_element_root = _get_data_structure_element(
            mock_user, self.fixture.data_structure_1
        )

        curate_data_structure_api.update_data_structure_root(
            self.fixture.data_structure_1,
            new_data_structure_element_root,
            mock_user,
        )

        self.assertEquals(
            self.fixture.data_structure_1.data_structure_element_root,
            new_data_structure_element_root,
        )
        self.assertEquals(
            DataStructureElement.get_by_id(new_data_structure_element_root.id),
            new_data_structure_element_root,
        )

        self.fixture.data_structure_1.delete()

        with self.assertRaises(DoesNotExist):
            curate_data_structure_api.get_by_id(
                self.fixture.data_structure_1.id, mock_user
            )
        with self.assertRaises(DoesNotExist):
            DataStructureElement.get_by_id(new_data_structure_element_root.id)


def _get_data_structure_element(user, data_structure):
    """Return a data structure element

    Args:
        user:
        data_structure:

    Returns:

    """
    data_structure_element = DataStructureElement(
        user=user,
        tag="tag",
        value="value",
        options={},
        data_structure=data_structure,
    )
    data_structure_element.save()
    return data_structure_element
